# FAQs, tips and tricks

1. [How do I verify the computational results?](#how-do-i-verify-the-computational-results)
2. [How can I find a good starting point for my PSPManalysis computation?](#how-can-i-find-a-good-starting-point-for-my-pspmanalysis-computation)
3. [Adaptive dynamics with direct, density-dependent interactions between individuals](#adaptive-dynamics-with-direct-density-dependent-interactions-between-individuals)
4. [Tweaking the compilation of models on Mac OS and Linux](#tweaking-the-compilation-of-models-on-mac-os-and-linux)

## 1. How do I verify the computational results?

Don't kid yourself, bugs are everywhere and therefore very likely also in your (and my!) code. So, I always insist on verifying my computations as much as possible. Here are some possible ways to do so:

**Analytical verification**

This is of course the best way to verify your computations, even though in most cases it might not be possible. If you can compute a solution analytically, it is the best way to check your problem implementation. You have a good chance to analytically verify your results, if you can compute the expected lifetime reproductive output (or R0) of an individual in your structured population. Even though the resulting expression for R0 may be complex, you can just use Maple, Matlab or R to evaluate whether the expression for R0 yields a value of 1, when you plug in the appropriate parameter values as well as the values of the environmental variables that you have computed with your PSPManalysis code. Similarly, once you have computed a solution point with your PSPManalysis implementation you can use the conditions for an equilibrium state of the environmental variables to assess that these conditions are indeed equal to 0 when you plug in all the computed values of environmental variables and population birth rates.

**Numerical/computationl verification?**

This is much harder. You could however compare the results from your PSPManalysis computations with results from numerical integrations over time of the model. Currently, the only way to do so is to use the [EBTtool](https://staff.fnwi.uva.nl/a.m.deroos/EBT/index.html) that I have developed previously and earlier than the PSPManalysis package. I am working on a better integration of the EBTtool with PSPManalysis, but for the time being the two packages need two separate implementations of the same model. 

One question I often encounter is how can you verify the results of the PSPManalysis computations, which yield equilibrium values, if the dynamics predicted by the EBTtool integrations show oscillatory behavior. An important fact to realise about population dynamic models is that with increasing mortality eventually a population will always go extinct. The general, mathemetical theory about bifurcations in dynamical systems guarantees that just before a transcritical bifurcation, to which an extinction threshold of a population corresponds, the dynamics of a system are necessarily stable, if the equilibrium state beyond the bifurcation point (that is at even higher mortality) corresponds to a stable equilibrium. Hence, a handy way to verify your PSPManalysis results with your EBTtool results is to compute the population dynamics at high mortality where the population is close to extinction. In this case both methods of calculating the population dynamics should yield exactly the same results. (*Do not go easy on yourself here: exactly the same should really mean exactly the same and not approximately the same. Difference of more than* 1.0E-04 *I would start to question*.)


## 2. How can I find a good starting point for my PSPManalysis computation?

Starting the analysis of a PSPM from some random initial values to search for the equilibrium values of environment variables and population birth rates is often not a very successful strategy. Most likely, the initial point will be too far off a solution point, which might cause the software not to converge to the solution. A better alternative is to start with a trivial equilibrium of the model, the value of which is known on beforehand. For example, it is biologically realistic to assume that at very high mortality or at very low food conditions a population is extinct. Such an extinct state is often easily characterized and hence provides a useful starting point.

## 3. Adaptive dynamics with direct, density-dependent interactions between individuals

In case the individuals in a structured populations are engaged in direct interactions with each other, carrying out adaptive dynamics analysis may become tricky. Consider as an example the case of cannibalism, where some larger or older individuals of the population forage on smaller/younger individuals of the same populations. This is one particular example of a direct, density-dependent interactions. 

Now consider that you are interested in the strength of this cannibalistic interactions and in particular you are interested in the evolutionary dynamics of this cannibalistic interaction strength. Let the parameter *β* be a measure of this cannibalistic interaction strength. Furthermore, let's assume for simplicity that the cannibalism is linear proportional to the number of cannibalistic individuals in the population, which I will indicate with *C*, and linear proportional to the number of victims in the population, which I will indicate with *V*. (*NB: if the strength of the cannibalistic interaction would scale with the body size of the cannibal, the variable C would of course represent the biomass of all cannibalistic individuals*). In short, at the population level the cannibalistic interactions would be represented with an expression *β C V*, whereas at the individual level cannibals derive an additional food intake from cannibalism that is proportional to *β V*, whereas victims of cannibals experience an additional mortality equal to *β C*.

In the context of adaptive dynamics, however, it is now important to realise that the cannibalistic interaction strength *β* is **a property or trait of the cannibal, but not of its victim**. This means that a mutant individual with a slightly different cannibalistic tendency and hence a slightly different value of *β = βm* experiences an additional food intake that is proportional to *βm V* with *V* the number of victims present in the *resident* population. In contrast, the mortality that this mutant individual experiences equals *βr C*, where *βr* is the cannibalistic intensity of the resident and *C* is the number of cannibalistic individuals in the resident population. Therefore, in the PSPManalysis code the parameter *β* should occur in the Development() function, but **should not occur** in the Mortality() function. If it would be used in the Mortality() function the program would use the mutant value of *β = βm* when calculating the mutant's mortality rate, as opposed to the resident value of *β = βr*. The solution here is to subsume the parameter *β* for its mortality impact into an environmental variable instead and define one equal to *E = β C*. In this way the mutant always experiences the (cannibalistic) environment as set by the resident.


## 4. Tweaking the compilation of models on Mac OS and Linux

Using **R** on Mac OS and Linux systems the compilation of a program file into an executable can be tailored by defining a number of variables, more specifically the variables `CC`, `CFLAGS` and `SHLIB_LDFLAGS`. These variables have to be defined in a file, called **Makevars**, which should be located in the directory `.R` in the user's home directory. The absolute path to this file is hence given by `${HOME}/.R/Makevars`. **R** looks for this particular file and reads its contents to override its default setting for compiling C code into an loadable module.

To get the package working properly on my Mac OS X (Sierra) system using **R** version 3.4.2, I have defined `CC`, `CFLAGS` and `SHLIB_LDFLAGS` as follows:

>            CC=/opt/local/bin/clang
>            CFLAGS = -fopenmp -Wall -mtune=core2 -O2 $(LTO)
>            SHLIB_LDFLAGS = -fopenmp -dynamiclib
>            SHLIB_LDFLAGS += -Wl,-headerpad_max_install_names  -undefined dynamic_lookup -single_module -multiply_defined suppress

The variable `CC` is set to the compiler that I use on my Mac (I checked the package both with the `clang` and `gcc` compiler). I do not use the `clang` compiler that is standard supplied with **XCode** on Mac OS X Sierra, as this version of `clang` is old and does not support parallelisation of the program via **OpenMP**. The `clang` and `gcc` compiler I use I have installed from **macports**. Parallelisation speeds up the computations significantly in case individuals of a particular population can be born with different states at birth. Furthermore, to make use of the automatic parallelisation offered by **OpenMP** the command-line argument `-fopenmp` is added to both the `CFLAGS` and the `SHLIB_LDFLAGS` variable. The parts of the `CFLAGS` and the `SHLIB_LDFLAGS` variable following the option `-fopenmp` I copied from the default values of the variables `CFLAGS` and `SHLIB_LDFLAGS`, as defined in the **R** distribution itself.

## Acknowledgments

![ERC logo](https://staff.fnwi.uva.nl/a.m.deroos/PSPManalysis/files/logo-erc-2.png)

The development of this software has been made possible by financial support from the European Research Council under the European Union's Seventh Framework Programme (FP/2007-2013) / ERC Grant Agreement No. 322814
